@@ -1,4 +1,4 @@
-/* Copyright (C) 2015-2023 Ben Collins <bcollins@maclara-llc.com>
+/* Copyright (C) 2015-2024 Ben Collins <bcollins@maclara-llc.com>
    This file is part of the JWT C Library
 
    SPDX-License-Identifier:  MPL-2.0
@@ -67,6 +67,8 @@ typedef enum jwt_alg {
 	JWT_ALG_PS256,
 	JWT_ALG_PS384,
 	JWT_ALG_PS512,
+	JWT_ALG_ES256K,
+	JWT_ALG_EDDSA,
 	JWT_ALG_TERM
 } jwt_alg_t;
 
@@ -98,6 +100,44 @@ typedef struct {
 /** Key provider - inspects the JWT to obtain the key used to verify the signature */
 typedef int (*jwt_key_p_t)(const jwt_t *, jwt_key_t *);
 
+
+/**
+ * @defgroup jwt_crypto JWT Crypto Operations
+ * Functions used to set and get which crypto operations are used
+ *
+ * LibJWT supports several crypto libaries, mainly "openssl" and "gnutls.
+ * By default, if enabled, "openssl" is used.
+ *
+ * NOTE: Changing the crypto operations is not thread safe. You must
+ * protect changing them with some sort of lock, including locking
+ * around usage of the operations themselves.
+ *
+ * ENVIRONMENT: You can set JWT_CRYPTO to the default operations you
+ * wish to use. If JWT_CRYPTO is invalid, an error message will be
+ * printed to the console when LibJWT is loaded by the application.
+ * @{
+ */
+
+/**
+ * Retrieve the name of the current crypto operations being used.
+ *
+ * @return name of the crypto operation set
+ */
+const char *jwt_get_crypto_ops(void);
+
+/**
+ * Set the crypto operations to the named set.
+ *
+ * The opname is one of the available operators in the compiled version
+ * of LibJWT. Most times, this is either "openssl" or "gnutls".
+ *
+ * @param opname the name of the crypto operation to set
+ * @return 0 on success, valid errno otherwise.
+ */
+int jwt_set_crypto_ops(const char *opname);
+
+/** @} */
+
 /**
  * @defgroup jwt_new JWT Object Creation
  * Functions used to create and destroy JWT objects.
@@ -127,7 +167,7 @@ typedef int (*jwt_key_p_t)(const jwt_t *, jwt_key_t *);
 JWT_EXPORT int jwt_new(jwt_t **jwt);
 
 /**
- * Verify an existing JWT and allocate a new JWT object from it.
+ * Decode an existing JWT and allocate a new JWT object from it.
  *
  * Decodes a JWT string and verifies the signature (if one is supplied).
  * If no signature is used (JWS, alg="none") or key is NULL, then no
@@ -149,6 +189,9 @@ JWT_EXPORT int jwt_new(jwt_t **jwt);
  *     non-encrypted token will be parsed without any checks for a valid
  *     signature, however, standard validation of the token is still
  *     performed.
+ *
+ * @warning You should not assume that decoding means your JWT is verified.
+ *     You should check the resulting jwt_t for proper algorithm and grants.
  */
 JWT_EXPORT int jwt_decode(jwt_t **jwt, const char *token,
 	                 const unsigned char *key, int key_len);
