@@ -1,6 +1,7 @@
-/* Copyright (C) 2015-2022 Ben Collins <bcollins@maclara-llc.com>
+/* Copyright (C) 2015-2024 Ben Collins <bcollins@maclara-llc.com>
    This file is part of the JWT C Library
 
+   SPDX-License-Identifier:  MPL-2.0
    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -15,7 +16,6 @@
 #include <jwt.h>
 
 #include "jwt-private.h"
-#include "config.h"
 
 /* Routines to support crypto in LibJWT using Windows crypto APIs. */
 
@@ -623,7 +623,7 @@ int jwt_verify_sha_hmac(jwt_t *jwt, const char *head, const char *sig)
 	jwt_base64uri_encode(pbB64);
 
 	/* And now... */
-	ret = strcmp(pbB64, sig) ? EINVAL : 0;
+	ret = jwt_strcmp(pbB64, sig) ? EINVAL : 0;
 
 jwt_verify_hmac_done:
 	if (pbHash)
@@ -677,6 +677,7 @@ int jwt_sign_sha_pem(jwt_t *jwt, char **out, unsigned int *len,
 
 	/* ECC */
 	case JWT_ALG_ES256:
+	case JWT_ALG_ES256K:
 		alg = BCRYPT_SHA256_ALGORITHM;
 		isECDSA = 1;
 		break;
@@ -873,6 +874,7 @@ int jwt_verify_sha_pem(jwt_t *jwt, const char *head, const char *sig_b64)
 
 	/* ECC */
 	case JWT_ALG_ES256:
+	case JWT_ALG_ES256K:
 		alg = BCRYPT_SHA256_ALGORITHM;
 		isECDSA = 1;
 		break;
