@@ -1,6 +1,7 @@
-/* Copyright (C) 2015-2022 Ben Collins <bcollins@maclara-llc.com>
+/* Copyright (C) 2015-2024 Ben Collins <bcollins@maclara-llc.com>
    This file is part of the JWT C Library
 
+   SPDX-License-Identifier:  MPL-2.0
    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -12,9 +13,77 @@
 
 #include <jwt.h>
 
+#ifdef HAVE_LIBB64
+#include <b64/cencode.h>
+#include <b64/cdecode.h>
+#else
+#include "cencode.h"
+#include "cdecode.h"
+#endif
+
 #include "jwt-private.h"
-#include "base64.h"
-#include "config.h"
+
+/* Library init functionality */
+static struct jwt_crypto_ops *jwt_ops_available[] = {
+#ifdef HAVE_OPENSSL
+	&jwt_openssl_ops,
+#endif
+#ifdef HAVE_GNUTLS
+	&jwt_gnutls_ops,
+#endif
+	NULL,
+};
+
+#ifdef HAVE_OPENSSL
+static struct jwt_crypto_ops *jwt_ops = &jwt_openssl_ops;
+#else
+static struct jwt_crypto_ops *jwt_ops = &jwt_gnutls_ops;
+#endif
+
+const char *jwt_get_crypto_ops(void)
+{
+	if (jwt_ops == NULL)
+		return "(unknown)";
+
+	return jwt_ops->name;
+}
+
+int jwt_set_crypto_ops(const char *opname)
+{
+	int i;
+
+	/* The user asked for something, let's give it a try */
+	for (i = 0; jwt_ops_available[i] != NULL; i++) {
+		if (strcmp(jwt_ops_available[i]->name, opname))
+			continue;
+
+		jwt_ops = jwt_ops_available[i];
+		return 0;
+	}
+
+	return EINVAL;
+}
+
+__attribute__((constructor)) void jwt_init()
+{
+	const char *opname = getenv("JWT_CRYPTO");
+
+	/* By default, we choose the top spot */
+	if (opname == NULL || opname[0] == '\0') {
+		jwt_ops = jwt_ops_available[0];
+		return;
+	}
+
+	/* Attempt to set ops */
+	if (jwt_set_crypto_ops(opname)) {
+		jwt_ops = jwt_ops_available[0];
+		fprintf(stderr, "LibJWT: No such crypto ops [%s], falling back to [%s]\n",
+			opname, jwt_ops->name);
+	}
+}
+
+/* Number of elements in an array */
+#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 
 static jwt_malloc_t pfn_malloc = NULL;
 static jwt_realloc_t pfn_realloc = NULL;
@@ -28,7 +97,7 @@ void *jwt_malloc(size_t size)
 	return malloc(size);
 }
 
-static void *jwt_realloc(void* ptr, size_t size)
+static void *jwt_realloc(void *ptr, size_t size)
 {
 	if (pfn_realloc)
 		return pfn_realloc(ptr, size);
@@ -59,10 +128,39 @@ static char *jwt_strdup(const char *str)
 	return result;
 }
 
+/* A time-safe strcmp function */
+int jwt_strcmp(const char *str1, const char *str2)
+{
+	/* Get the LONGEST length */
+	int len1 = strlen(str1);
+	int len2 = strlen(str2);
+	int len_max = len1 >= len2 ? len1 : len2;
+
+	int i, ret = 0;
+
+	/* Iterate the entire longest string no matter what. Only testing
+	 * the shortest string would still allow attacks for
+	 * "a" == "aKJSDHkjashaaHJASJ", adding a character each time one
+	 * is found. */
+	for (i = 0; i < len_max; i++) {
+		char c1, c2;
+
+		c1 = (i < len1) ? str1[i] : 0;
+		c2 = (i < len2) ? str2[i] : 0;
+
+		ret |= c1 ^ c2;
+	}
+
+	/* Don't forget to check length */
+	ret |= len1 ^ len2;
+
+	return ret;
+}
+
 static void *jwt_calloc(size_t nmemb, size_t size)
 {
 	size_t total_size;
-	void* ptr;
+	void *ptr;
 
 	total_size = nmemb * size;
 	if (!total_size)
@@ -94,6 +192,8 @@ const char *jwt_alg_str(jwt_alg_t alg)
 		return "RS512";
 	case JWT_ALG_ES256:
 		return "ES256";
+	case JWT_ALG_ES256K:
+		return "ES256K";
 	case JWT_ALG_ES384:
 		return "ES384";
 	case JWT_ALG_ES512:
@@ -104,6 +204,8 @@ const char *jwt_alg_str(jwt_alg_t alg)
 		return "PS384";
 	case JWT_ALG_PS512:
 		return "PS512";
+	case JWT_ALG_EDDSA:
+		return "EDDSA";
 	default:
 		return NULL;
 	}
@@ -130,6 +232,8 @@ jwt_alg_t jwt_str_alg(const char *alg)
 		return JWT_ALG_RS512;
 	else if (!strcmp(alg, "ES256"))
 		return JWT_ALG_ES256;
+	else if (!strcmp(alg, "ES256K"))
+		return JWT_ALG_ES256K;
 	else if (!strcmp(alg, "ES384"))
 		return JWT_ALG_ES384;
 	else if (!strcmp(alg, "ES512"))
@@ -140,6 +244,8 @@ jwt_alg_t jwt_str_alg(const char *alg)
 		return JWT_ALG_PS384;
 	else if (!strcmp(alg, "PS512"))
 		return JWT_ALG_PS512;
+	else if (!strcmp(alg, "EDDSA"))
+		return JWT_ALG_EDDSA;
 
 	return JWT_ALG_INVAL;
 }
@@ -191,6 +297,9 @@ int jwt_set_alg(jwt_t *jwt, jwt_alg_t alg, const unsigned char *key, int len)
 
 jwt_alg_t jwt_get_alg(const jwt_t *jwt)
 {
+	if (jwt == NULL)
+		return JWT_ALG_INVAL;
+
 	return jwt->alg;
 }
 
@@ -290,11 +399,10 @@ static const char *get_js_string(json_t *js, const char *key)
 
 	js_val = json_object_get(js, key);
 	if (js_val) {
-		if (json_typeof(js_val) == JSON_STRING) {
+		if (json_typeof(js_val) == JSON_STRING)
 			val = json_string_value(js_val);
-		} else {
+		else
 			errno = EINVAL;
-		}
 	} else {
 		errno = ENOENT;
 	}
@@ -309,11 +417,10 @@ static long get_js_int(json_t *js, const char *key)
 
 	js_val = json_object_get(js, key);
 	if (js_val) {
-		if (json_typeof(js_val) == JSON_INTEGER) {
+		if (json_typeof(js_val) == JSON_INTEGER)
 			val = (long)json_integer_value(js_val);
-		} else {
+		else
 			errno = EINVAL;
-		}
 	} else {
 		errno = ENOENT;
 	}
@@ -329,14 +436,14 @@ static int get_js_bool(json_t *js, const char *key)
 	js_val = json_object_get(js, key);
 	if (js_val) {
 		switch (json_typeof(js_val)) {
-			case JSON_TRUE:
-				val = 1;
-				break;
-			case JSON_FALSE:
-				val = 0;
-				break;
-			default:
-				errno = EINVAL;
+		case JSON_TRUE:
+			val = 1;
+			break;
+		case JSON_FALSE:
+			val = 0;
+			break;
+		default:
+			errno = EINVAL;
 		}
 	} else {
 		errno = ENOENT;
@@ -344,14 +451,17 @@ static int get_js_bool(json_t *js, const char *key)
 	return val;
 }
 
-void *jwt_b64_decode(const char *src, int *ret_len)
+void *jwt_base64uri_decode(const char *src, int *ret_len)
 {
+	base64_decodestate state;
 	void *buf;
 	char *new;
 	int len, i, z;
 
 	/* Decode based on RFC-4648 URI safe encoding. */
 	len = (int)strlen(src);
+	/* When reversing the URI cleanse, we can possibly add up
+	 * to 3 '=' characters to replace the missing padding. */
 	new = alloca(len + 4);
 	if (!new)
 		return NULL;
@@ -375,23 +485,30 @@ void *jwt_b64_decode(const char *src, int *ret_len)
 	}
 	new[i] = '\0';
 
-	buf = jwt_malloc(i);
+	/* Now we have a standard base64 encoded string. */
+	buf = jwt_malloc(base64_decode_maxlength(len) + 1);
 	if (buf == NULL)
 		return NULL;
 
-	*ret_len = jwt_Base64decode(buf, new);
+	base64_init_decodestate(&state);
+	*ret_len = base64_decode_block(new, strlen(new), buf, &state);
+
+	if (*ret_len == 0) {
+		jwt_freemem(buf);
+		buf = NULL;
+	}
 
 	return buf;
 }
 
 
-static json_t *jwt_b64_decode_json(char *src)
+static json_t *jwt_base64uri_decode_to_json(char *src)
 {
 	json_t *js;
 	char *buf;
 	int len;
 
-	buf = jwt_b64_decode(src, &len);
+	buf = jwt_base64uri_decode(src, &len);
 
 	if (buf == NULL)
 		return NULL;
@@ -405,27 +522,45 @@ static json_t *jwt_b64_decode_json(char *src)
 	return js;
 }
 
-void jwt_base64uri_encode(char *str)
+int jwt_base64uri_encode(char **_dst, const char *plain, int plain_len)
 {
-	int len = (int)strlen(str);
-	int i, t;
+	base64_encodestate state;
+	int len, i;
+	char *dst;
 
-	for (i = t = 0; i < len; i++) {
-		switch (str[i]) {
+	base64_init_encodestate(&state);
+	/* Ensure no newlines are emitted into the string */
+	state.chars_per_line = 0;
+
+	len = base64_encode_length(plain_len, &state);
+	*_dst = jwt_malloc(len + 1);
+	if (*_dst == NULL)
+		return -ENOMEM;
+	dst = *_dst;
+
+	/* First, a normal base64 encoding */
+	len = base64_encode_block(plain, plain_len, dst, &state);
+	len += base64_encode_blockend(dst + len, &state);
+        dst[len] = 0;
+
+	/* Now for the URI encoding */
+	for (i = 0; i < len; i++) {
+		switch (dst[i]) {
 		case '+':
-			str[t++] = '-';
+			dst[i] = '-';
 			break;
 		case '/':
-			str[t++] = '_';
+			dst[i] = '_';
 			break;
 		case '=':
+			dst[i] = '\0';
 			break;
-		default:
-			str[t++] = str[i];
 		}
 	}
 
-	str[t] = '\0';
+	dst[i] = '\0';
+
+	return i;
 }
 
 static int jwt_sign(jwt_t *jwt, char **out, unsigned int *len, const char *str, unsigned int str_len)
@@ -435,23 +570,27 @@ static int jwt_sign(jwt_t *jwt, char **out, unsigned int *len, const char *str,
 	case JWT_ALG_HS256:
 	case JWT_ALG_HS384:
 	case JWT_ALG_HS512:
-		return jwt_sign_sha_hmac(jwt, out, len, str, str_len);
+		return jwt_ops->sign_sha_hmac(jwt, out, len, str, str_len);
 
 	/* RSA */
 	case JWT_ALG_RS256:
 	case JWT_ALG_RS384:
 	case JWT_ALG_RS512:
 
-	/* PS */
+	/* RSA-PSS */
 	case JWT_ALG_PS256:
 	case JWT_ALG_PS384:
 	case JWT_ALG_PS512:
 
 	/* ECC */
 	case JWT_ALG_ES256:
+	case JWT_ALG_ES256K:
 	case JWT_ALG_ES384:
 	case JWT_ALG_ES512:
-		return jwt_sign_sha_pem(jwt, out, len, str, str_len);
+
+	/* EdDSA */
+	case JWT_ALG_EDDSA:
+		return jwt_ops->sign_sha_pem(jwt, out, len, str, str_len);
 
 	/* You wut, mate? */
 	default:
@@ -466,23 +605,27 @@ static int jwt_verify(jwt_t *jwt, const char *head, unsigned int head_len, const
 	case JWT_ALG_HS256:
 	case JWT_ALG_HS384:
 	case JWT_ALG_HS512:
-		return jwt_verify_sha_hmac(jwt, head, head_len, sig);
+		return jwt_ops->verify_sha_hmac(jwt, head, head_len, sig);
 
 	/* RSA */
 	case JWT_ALG_RS256:
 	case JWT_ALG_RS384:
 	case JWT_ALG_RS512:
 
-	/* PS */
+	/* RSA-PSS */
 	case JWT_ALG_PS256:
 	case JWT_ALG_PS384:
 	case JWT_ALG_PS512:
 
 	/* ECC */
 	case JWT_ALG_ES256:
+	case JWT_ALG_ES256K:
 	case JWT_ALG_ES384:
 	case JWT_ALG_ES512:
-		return jwt_verify_sha_pem(jwt, head, head_len, sig);
+
+	/* EdDSA */
+	case JWT_ALG_EDDSA:
+		return jwt_ops->verify_sha_pem(jwt, head, head_len, sig);
 
 	/* You wut, mate? */
 	default:
@@ -497,7 +640,7 @@ static int jwt_parse_body(jwt_t *jwt, char *body)
 		jwt->grants = NULL;
 	}
 
-	jwt->grants = jwt_b64_decode_json(body);
+	jwt->grants = jwt_base64uri_decode_to_json(body);
 	if (!jwt->grants)
 		return EINVAL;
 
@@ -513,37 +656,39 @@ static int jwt_parse_head(jwt_t *jwt, char *head)
 		jwt->headers = NULL;
 	}
 
-	jwt->headers = jwt_b64_decode_json(head);
+	jwt->headers = jwt_base64uri_decode_to_json(head);
 	if (!jwt->headers)
 		return EINVAL;
 
 	alg = get_js_string(jwt->headers, "alg");
 	jwt->alg = jwt_str_alg(alg);
-	if (jwt->alg == JWT_ALG_INVAL) {
+	if (jwt->alg == JWT_ALG_INVAL)
 		return EINVAL;
-	}
 
 	return 0;
 }
 
-static int jwt_verify_head(jwt_t *jwt)
+/**
+ * @brief Smoke test to save the user from themselves.
+ */
+static int jwt_verify_alg(jwt_t *jwt)
 {
 	int ret = 0;
 
-	if (jwt->alg != JWT_ALG_NONE) {
-		if (jwt->key) {
-			if (jwt->key_len <= 0)
-				ret = EINVAL;
-		} else {
-			jwt_scrub_key(jwt);
-		}
-	} else {
-		/* If alg is NONE, there should not be a key */
-		if (jwt->key) {
+	if (jwt->alg == JWT_ALG_NONE) {
+		/* If the user gave us a key but the JWT has alg = none,
+		 * then we shouldn't even proceed. */
+		if (jwt->key || jwt->key_len)
 			ret = EINVAL;
-		}
+	} else if (!(jwt->key && (jwt->key_len > 0))) {
+		/* If alg != none, then we should have a key to use */
+		ret = EINVAL;
 	}
 
+	/* Releive ourselves of the burden of this secret. */
+	if (ret)
+		jwt_scrub_key(jwt);
+
 	return ret;
 }
 
@@ -582,19 +727,17 @@ static int jwt_parse(jwt_t **jwt, const char *token, unsigned int *len)
 	/* Now that we have everything split up, let's check out the
 	 * header. */
 	ret = jwt_new(&new);
-	if (ret) {
+	if (ret)
 		goto parse_done;
-	}
 
-	if ((ret = jwt_parse_head(new, head))) {
+	if ((ret = jwt_parse_head(new, head)))
 		goto parse_done;
-	}
 
 	ret = jwt_parse_body(new, body);
 parse_done:
 	if (ret) {
 		jwt_free(new);
-        	*jwt = NULL;
+		*jwt = NULL;
 	} else {
 		*jwt = new;
 		*len = sig - head;
@@ -609,12 +752,50 @@ static int jwt_copy_key(jwt_t *jwt, const unsigned char *key, int key_len)
 {
 	int ret = 0;
 
-	if (key_len) {
-		jwt->key = jwt_malloc(key_len);
-		if (jwt->key == NULL)
-			return ENOMEM;
-		memcpy(jwt->key, key, key_len);
-		jwt->key_len = key_len;
+	if (!key_len)
+		return 0;
+
+	/* Always allocate one extra byte. For PEM, it ensures against
+	 * not having a nil at the end (although all crypto backends
+	 * should honor length), and for binary keys, it wont hurt
+	 * because we use key_len for those operations. */
+	jwt->key = jwt_malloc(key_len + 1);
+	if (jwt->key == NULL)
+		return ENOMEM;
+
+	jwt->key[key_len] = '\0';
+
+	memcpy(jwt->key, key, key_len);
+	jwt->key_len = key_len;
+
+	return ret;
+}
+
+static int jwt_decode_complete(jwt_t **jwt, const unsigned char *key, int key_len,
+			       const char *token, unsigned int payload_len)
+{
+	int ret = EINVAL;
+	jwt_t *new = *jwt;
+
+	/* Copy the key over for verify_alg. */
+	ret = jwt_copy_key(new, key, key_len);
+	if (ret)
+		goto decode_done;
+
+	ret = jwt_verify_alg(new);
+	if (ret)
+		goto decode_done;
+
+	/* Check the signature, if needed. */
+	if (new->alg != JWT_ALG_NONE) {
+		const char *sig = token + (payload_len + 1);
+		ret = jwt_verify(new, token, payload_len, sig);
+	}
+
+decode_done:
+	if (ret) {
+		jwt_free(new);
+		*jwt = NULL;
 	}
 
 	return ret;
@@ -623,85 +804,59 @@ static int jwt_copy_key(jwt_t *jwt, const unsigned char *key, int key_len)
 int jwt_decode(jwt_t **jwt, const char *token, const unsigned char *key,
 	       int key_len)
 {
-	jwt_t *new = NULL;
-	int ret = EINVAL;
+	int ret;
 	unsigned int payload_len;
 
-	ret = jwt_parse(jwt, token, &payload_len);
-	if (ret) {
+	if (jwt == NULL)
+		return EINVAL;
+
+	if ((ret = jwt_parse(jwt, token, &payload_len)))
 		return ret;
-	}
-	new = *jwt;
 
-	/* Copy the key over for verify_head. */
-	ret = jwt_copy_key(new, key, key_len);
-	if (ret)
-		goto decode_done;
-
-	ret = jwt_verify_head(new);
-	if (ret)
-		goto decode_done;
-
-	/* Check the signature, if needed. */
-	if (new->alg != JWT_ALG_NONE) {
-		const char *sig = token + (payload_len + 1);
-		ret = jwt_verify(new, token, payload_len, sig);
-	} else {
-		ret = 0;
-	}
-
-decode_done:
-	if (ret) {
-		jwt_free(new);
-		*jwt = NULL;
-	}
-
-	return ret;
+	return jwt_decode_complete(jwt, key, key_len, token, payload_len);
 }
 
-int jwt_decode_2(jwt_t **jwt, const char *token, jwt_key_p_t key_provider) 
+int jwt_decode_2(jwt_t **jwt, const char *token, jwt_key_p_t key_provider)
 {
-	jwt_t *new = NULL;
-	int ret = EINVAL;
+	int ret;
 	unsigned int payload_len;
-	jwt_key_t key;
+	jwt_key_t key = { NULL, 0 };
+	jwt_t *new = NULL;
 
-	ret = jwt_parse(jwt, token, &payload_len);
-	if (ret) {
-		return ret;
-	}
-	new = *jwt;
+	if (jwt == NULL)
+		return EINVAL;
+	*jwt = NULL;
 
-	/* Obtain the key. */
-	if (new->alg != JWT_ALG_NONE) {
-		ret = key_provider(new, &key);
-		if (ret) {
-			goto decode_done;
-		}
-		ret = jwt_copy_key(new, key.jwt_key, key.jwt_key_len);
-		if (ret)
-			goto decode_done;
-	}
-
-	ret = jwt_verify_head(new);
+	ret = jwt_parse(&new, token, &payload_len);
 	if (ret)
-		goto decode_done;
+		return ret;
 
-	/* Check the signature, if needed. */
-	if (new->alg != JWT_ALG_NONE) {
-		ret = jwt_verify(new, token, payload_len,
-				 token + (payload_len + 1));
-	} else {
-		ret = 0;
+	if (key_provider) {
+		/* The previous code trusted the JWT alg too much. If it was
+		 * NONE, then it wouldn't even bother calling the cb.
+		 *
+		 * We also had some test cases that called this func with no
+		 * key_provider and exptected it to work. True, this code
+		 * allowed for that. My gut tells me that should never have
+		 * been the case.
+		 *
+		 * For one, the previous code didn't check for NULL, so if
+		 * you got a key that wasn't alg == none, instant SEGV.
+		 *
+		 * However, since this func is getting deprecated, we'll
+		 * just let that case be like calling jwt_decode()
+		 */
+		ret = key_provider(new, &key);
 	}
 
-decode_done:
 	if (ret) {
 		jwt_free(new);
-		*jwt = NULL;
+		return ret;
 	}
 
-	return ret;
+	*jwt = new;
+
+	return jwt_decode_complete(jwt, key.jwt_key, key.jwt_key_len, token, payload_len);
 }
 
 const char *jwt_get_grant(jwt_t *jwt, const char *grant)
@@ -972,11 +1127,10 @@ static int __append_str(char **buf, const char *str)
 {
 	char *new;
 
-	if (*buf == NULL) {
+	if (*buf == NULL)
 		new = jwt_calloc(1, strlen(str) + 1);
-	} else {
+	else
 		new = jwt_realloc(*buf, strlen(*buf) + strlen(str) + 1);
-	}
 
 	if (new == NULL)
 		return ENOMEM;
@@ -992,7 +1146,7 @@ static int __append_str(char **buf, const char *str)
 	int ret = __append_str(__buf, __str);	\
 	if (ret)				\
 		return ret;			\
-} while(0)
+} while (0)
 
 static int write_js(const json_t *js, char **buf, int pretty)
 {
@@ -1027,12 +1181,11 @@ static int jwt_write_head(jwt_t *jwt, char **buf, int pretty)
 
 		/* Only add default 'typ' header if it has not been defined,
 		 * allowing for any value of it. This allows for signaling
-		 * of application specific extentions to JWT, such as PASSporT,
+		 * of application specific extensions to JWT, such as PASSporT,
 		 * RFC 8225. */
 		if ((ret = jwt_add_header(jwt, "typ", "JWT"))) {
-			if (ret != EEXIST) {
+			if (ret != EEXIST)
 				return ret;
-			}
 		}
 	}
 
@@ -1065,6 +1218,25 @@ static int jwt_dump(jwt_t *jwt, char **buf, int pretty)
 	return ret;
 }
 
+char *jwt_dump_grants_str(jwt_t *jwt, int pretty)
+{
+	char *out = NULL;
+	int err;
+
+	errno = 0;
+
+	err = jwt_write_body(jwt, &out, pretty);
+
+	if (err) {
+		errno = err;
+		if (out)
+			jwt_freemem(out);
+		out = NULL;
+	}
+
+	return out;
+}
+
 int jwt_dump_fp(jwt_t *jwt, FILE *fp, int pretty)
 {
 	char *out = NULL;
@@ -1102,7 +1274,7 @@ char *jwt_dump_str(jwt_t *jwt, int pretty)
 
 static int jwt_encode(jwt_t *jwt, char **out)
 {
-	char *buf = NULL, *head, *body, *sig;
+	char *buf = NULL, *head = NULL, *body = NULL, *sig;
 	int ret, head_len, body_len;
 	unsigned int sig_len;
 
@@ -1119,12 +1291,13 @@ static int jwt_encode(jwt_t *jwt, char **out)
 		jwt_freemem(buf);
 		return ENOMEM;
 	}
-	jwt_Base64encode(head, buf, (int)strlen(buf));
-	head_len = (int)strlen(head);
-
+	head_len = jwt_base64uri_encode(&head, buf, (int)strlen(buf));
 	jwt_freemem(buf);
 	buf = NULL;
 
+	if (head_len <= 0)
+		return -head_len;
+
 	/* Now the body. */
 	ret = jwt_write_body(jwt, &buf, 0);
 	if (ret) {
@@ -1133,28 +1306,30 @@ static int jwt_encode(jwt_t *jwt, char **out)
 		return ret;
 	}
 
-	body = alloca(strlen(buf) * 2);
-	if (body == NULL) {
-		jwt_freemem(buf);
-		return ENOMEM;
-	}
-	jwt_Base64encode(body, buf, (int)strlen(buf));
-	body_len = (int)strlen(body);
-
+	body_len = jwt_base64uri_encode(&body, buf, (int)strlen(buf));
 	jwt_freemem(buf);
 	buf = NULL;
 
-	jwt_base64uri_encode(head);
-	jwt_base64uri_encode(body);
+	if (body_len <= 0) {
+		jwt_freemem(head);
+		return -body_len;
+	}
 
 	/* Allocate enough to reuse as b64 buffer. */
 	buf = jwt_malloc(head_len + body_len + 2);
-	if (buf == NULL)
+	if (buf == NULL) {
+		jwt_freemem(head);
+		jwt_freemem(body);
 		return ENOMEM;
+	}
+
 	strcpy(buf, head);
 	strcat(buf, ".");
 	strcat(buf, body);
 
+	jwt_freemem(head);
+	jwt_freemem(body);
+
 	ret = __append_str(out, buf);
 	if (ret == 0)
 		ret = __append_str(out, ".");
@@ -1172,21 +1347,16 @@ static int jwt_encode(jwt_t *jwt, char **out)
 	/* Now the signature. */
 	ret = jwt_sign(jwt, &sig, &sig_len, buf, strlen(buf));
 	jwt_freemem(buf);
+	buf = NULL;
 
 	if (ret)
 		return ret;
 
-	buf = jwt_malloc(sig_len * 2);
-	if (buf == NULL) {
-		jwt_freemem(sig);
-		return ENOMEM;
-	}
-
-	jwt_Base64encode(buf, sig, sig_len);
-
+	ret = jwt_base64uri_encode(&buf, sig, sig_len);
 	jwt_freemem(sig);
+	if (ret <= 0)
+		return -ret;
 
-	jwt_base64uri_encode(buf);
 	ret = __append_str(out, buf);
 	jwt_freemem(buf);
 
@@ -1244,7 +1414,7 @@ int jwt_set_alloc(jwt_malloc_t pmalloc, jwt_realloc_t prealloc, jwt_free_t pfree
 	return 0;
 }
 
-void jwt_get_alloc(jwt_malloc_t *pmalloc, jwt_realloc_t* prealloc, jwt_free_t *pfree)
+void jwt_get_alloc(jwt_malloc_t *pmalloc, jwt_realloc_t *prealloc, jwt_free_t *pfree)
 {
 	if (pmalloc)
 		*pmalloc = pfn_malloc;
@@ -1491,7 +1661,7 @@ int jwt_valid_del_grants(jwt_valid_t *jwt_valid, const char *grant)
 #define _SET_AND_RET(__v, __e) do {	\
 	__v->status |= __e;		\
 	return __v->status;		\
-} while(0)
+} while (0)
 
 unsigned int jwt_validate(jwt_t *jwt, jwt_valid_t *jwt_valid)
 {
@@ -1556,3 +1726,54 @@ unsigned int jwt_validate(jwt_t *jwt, jwt_valid_t *jwt_valid)
 
 	return jwt_valid->status;
 }
+
+typedef struct {
+	int error;
+	char *str;
+} jwt_exception_dict_t;
+
+static jwt_exception_dict_t jwt_exceptions[] = {
+	/* { JWT_VALIDATION_SUCCESS, "SUCCESS" }, */
+	{ JWT_VALIDATION_ERROR, "general failures" },
+	{ JWT_VALIDATION_ALG_MISMATCH, "algorithm mismatch" },
+	{ JWT_VALIDATION_EXPIRED, "token expired" },
+	{ JWT_VALIDATION_TOO_NEW, "token future dated" },
+	{ JWT_VALIDATION_ISS_MISMATCH, "issuer mismatch" },
+	{ JWT_VALIDATION_SUB_MISMATCH, "subject mismatch" },
+	{ JWT_VALIDATION_AUD_MISMATCH, "audience mismatch" },
+	{ JWT_VALIDATION_GRANT_MISSING, "grant missing" },
+	{ JWT_VALIDATION_GRANT_MISMATCH, "grant mismatch" },
+};
+
+char *jwt_exception_str(unsigned int exceptions)
+{
+	int rc, i;
+	char *str = NULL;
+
+	if (exceptions == JWT_VALIDATION_SUCCESS) {
+		if ((rc = __append_str(&str, "success")))
+			goto fail;
+		return str;
+	}
+
+	for (i = 0; i < ARRAY_SIZE(jwt_exceptions); i++) {
+		if (!(jwt_exceptions[i].error & exceptions))
+			continue;
+
+		if (str && (rc = __append_str(&str, ", ")))
+			goto fail;
+
+		if ((rc = __append_str(&str, jwt_exceptions[i].str)))
+			goto fail;
+	}
+
+	/* check if none of the exceptions matched? */
+	if (!str && (rc = __append_str(&str, "unknown exceptions")))
+		goto fail;
+
+	return str;
+fail:
+	errno = rc;
+	jwt_freemem(str);
+	return NULL;
+}
