@@ -6,32 +6,7 @@
 #include <errno.h>
 #include <time.h>
 
-#include <check.h>
-
-#include <jwt.h>
-
-/* Constant time to make tests consistent. */
-#define TS_CONST	1475980545L
-
-/* Macro to allocate a new JWT with checks. */
-#define ALLOC_JWT(__jwt) do {		\
-	int __ret = jwt_new(__jwt);	\
-	ck_assert_int_eq(__ret, 0);	\
-	ck_assert_ptr_ne(__jwt, NULL);	\
-} while(0)
-
-/* Older check doesn't have this. */
-#ifndef ck_assert_ptr_ne
-#define ck_assert_ptr_ne(X, Y) ck_assert(X != Y)
-#define ck_assert_ptr_eq(X, Y) ck_assert(X == Y)
-#endif
-
-#ifndef ck_assert_int_gt
-#define ck_assert_int_gt(X, Y) ck_assert(X > Y)
-#endif
-
-static unsigned char key[16384];
-static size_t key_len;
+#include "jwt_tests.h"
 
 static const char jwt_rs256_2048[] = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.ey"
 	"JpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLm1hY2xhcmEtbGxjLmNvbSIsInJlZi"
@@ -84,95 +59,16 @@ static const char jwt_rs256_invalid[] = "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9"
 	".eyJpYXQiOjE0NzU5ODA1NDUsImlzcyI6ImZpbGVzLmN5cGhyZS5jb20iLCJyZWYiOiJ"
 	"YWFhYLVlZWVktWlpaWi1BQUFBLUNDQ0MiLCJzdWIiOiJ1c2VyMCJ9.IAmCornholio";
 
-static void read_key(const char *key_file)
-{
-	FILE *fp = fopen(key_file, "r");
-	char *key_path;
-	int ret = 0;
-
-	ret = asprintf(&key_path, KEYDIR "/%s", key_file);
-	ck_assert_int_gt(ret, 0);
-
-	fp = fopen(key_path, "r");
-	ck_assert_ptr_ne(fp, NULL);
-
-	jwt_free_str(key_path);
-
-	key_len = fread(key, 1, sizeof(key), fp);
-	ck_assert_int_ne(key_len, 0);
-
-	ck_assert_int_eq(ferror(fp), 0);
-
-	fclose(fp);
-
-	key[key_len] = '\0';
-}
-
-static void __test_alg_key(const char *key_file, const char *jwt_str,
-			   const jwt_alg_t alg)
-{
-	jwt_t *jwt = NULL;
-	int ret = 0;
-	char *out;
-
-	ALLOC_JWT(&jwt);
-
-	read_key(key_file);
-
-	ret = jwt_add_grant(jwt, "iss", "files.maclara-llc.com");
-	ck_assert_int_eq(ret, 0);
-
-	ret = jwt_add_grant(jwt, "sub", "user0");
-	ck_assert_int_eq(ret, 0);
-
-	ret = jwt_add_grant(jwt, "ref", "XXXX-YYYY-ZZZZ-AAAA-CCCC");
-	ck_assert_int_eq(ret, 0);
-
-	ret = jwt_add_grant_int(jwt, "iat", TS_CONST);
-	ck_assert_int_eq(ret, 0);
-
-	ret = jwt_set_alg(jwt, alg, key, key_len);
-	ck_assert_int_eq(ret, 0);
-
-	out = jwt_encode_str(jwt);
-	ck_assert_ptr_ne(out, NULL);
-
-	ck_assert_str_eq(out, jwt_str);
-
-	jwt_free_str(out);
-	jwt_free(jwt);
-}
-
-static void __verify_alg_key(const char *key_file, const char *jwt_str,
-			     const jwt_alg_t alg)
-{
-	jwt_valid_t *jwt_valid = NULL;
-	jwt_t *jwt = NULL;
-	int ret = 0;
-
-	read_key(key_file);
-
-	ret = jwt_decode(&jwt, jwt_str, key, key_len);
-	ck_assert_int_eq(ret, 0);
-	ck_assert_ptr_nonnull(jwt);
-
-	jwt_valid_new(&jwt_valid, alg);
-
-	ret = jwt_validate(jwt, jwt_valid);
-	ck_assert_int_eq(JWT_VALIDATION_SUCCESS, ret);
-
-	jwt_valid_free(jwt_valid);
-	jwt_free(jwt);
-}
-
 START_TEST(test_jwt_encode_rs256)
 {
-	__test_alg_key("rsa_key_2048.pem", jwt_rs256_2048, JWT_ALG_RS256);
+	SET_OPS();
+	__compare_alg_key("rsa_key_2048.pem", jwt_rs256_2048, JWT_ALG_RS256);
 }
 END_TEST
 
 START_TEST(test_jwt_verify_rs256)
 {
+	SET_OPS();
 	__verify_alg_key("rsa_key_2048-pub.pem", jwt_rs256_2048, JWT_ALG_RS256);
 }
 END_TEST
@@ -183,6 +79,8 @@ START_TEST(test_jwt_validate_rs256)
 	jwt_valid_t *jwt_valid = NULL;
 	int ret = 0;
 
+	SET_OPS();
+
 	read_key("rsa_key_2048-pub.pem");
 
 	ret = jwt_decode(&jwt, jwt_rs256_2048, key, key_len);
@@ -207,24 +105,28 @@ END_TEST
 
 START_TEST(test_jwt_encode_rs384)
 {
-	__test_alg_key("rsa_key_4096.pem", jwt_rs384_4096, JWT_ALG_RS384);
+	SET_OPS();
+	__compare_alg_key("rsa_key_4096.pem", jwt_rs384_4096, JWT_ALG_RS384);
 }
 END_TEST
 
 START_TEST(test_jwt_verify_rs384)
 {
+	SET_OPS();
 	__verify_alg_key("rsa_key_4096-pub.pem", jwt_rs384_4096, JWT_ALG_RS384);
 }
 END_TEST
 
 START_TEST(test_jwt_encode_rs512)
 {
-	__test_alg_key("rsa_key_8192.pem", jwt_rs512_8192, JWT_ALG_RS512);
+	SET_OPS();
+	__compare_alg_key("rsa_key_8192.pem", jwt_rs512_8192, JWT_ALG_RS512);
 }
 END_TEST
 
 START_TEST(test_jwt_verify_rs512)
 {
+	SET_OPS();
 	__verify_alg_key("rsa_key_8192-pub.pem", jwt_rs512_8192, JWT_ALG_RS512);
 }
 END_TEST
@@ -246,6 +148,7 @@ static const char jwt_rsa_i37[] = "eyJraWQiOiJkWUoxTDVnbWd0eDlWVU9xbVpyd2F6cW"
 
 START_TEST(test_jwt_verify_rsa_i37)
 {
+	SET_OPS();
 	__verify_alg_key("rsa_key_i37-pub.pem", jwt_rsa_i37, JWT_ALG_RS256);
 }
 END_TEST
@@ -256,6 +159,8 @@ START_TEST(test_jwt_encode_rsa_with_ec)
 	int ret = 0;
 	char *out;
 
+	SET_OPS();
+
 	ALLOC_JWT(&jwt);
 
 	read_key("ec_key_secp384r1.pem");
@@ -288,6 +193,8 @@ START_TEST(test_jwt_verify_invalid_token)
 	jwt_t *jwt = NULL;
 	int ret = 0;
 
+	SET_OPS();
+
 	read_key("rsa_key_2048.pem");
 
 	ret = jwt_decode(&jwt, jwt_rs256_invalid, key, JWT_ALG_RS512);
@@ -301,6 +208,8 @@ START_TEST(test_jwt_verify_invalid_alg)
 	jwt_t *jwt = NULL;
 	int ret = 0;
 
+	SET_OPS();
+
 	read_key("rsa_key_2048.pem");
 
 	ret = jwt_decode(&jwt, jwt_rs256_2048, key, JWT_ALG_RS512);
@@ -314,6 +223,8 @@ START_TEST(test_jwt_verify_invalid_cert)
 	jwt_t *jwt = NULL;
 	int ret = 0;
 
+	SET_OPS();
+
 	read_key("rsa_key_8192-pub.pem");
 
 	ret = jwt_decode(&jwt, jwt_rs256_2048, key, JWT_ALG_RS256);
@@ -327,6 +238,8 @@ START_TEST(test_jwt_verify_invalid_cert_file)
 	jwt_t *jwt = NULL;
 	int ret = 0;
 
+	SET_OPS();
+
 	read_key("rsa_key_invalid-pub.pem");
 
 	ret = jwt_decode(&jwt, jwt_rs256_2048, key, JWT_ALG_RS256);
@@ -341,6 +254,8 @@ START_TEST(test_jwt_encode_invalid_key)
 	int ret = 0;
 	char *out = NULL;
 
+	SET_OPS();
+
 	ALLOC_JWT(&jwt);
 
 	read_key("rsa_key_invalid.pem");
@@ -367,29 +282,30 @@ START_TEST(test_jwt_encode_invalid_key)
 }
 END_TEST
 
-static Suite *libjwt_suite(void)
+static Suite *libjwt_suite(const char *title)
 {
 	Suite *s;
 	TCase *tc_core;
+	int i = ARRAY_SIZE(jwt_test_ops) - 1;
 
-	s = suite_create("LibJWT RSA Sign/Verify");
+	s = suite_create(title);
 
 	tc_core = tcase_create("jwt_rsa");
 
-	tcase_add_test(tc_core, test_jwt_encode_rs256);
-	tcase_add_test(tc_core, test_jwt_verify_rs256);
-	tcase_add_test(tc_core, test_jwt_validate_rs256);
-	tcase_add_test(tc_core, test_jwt_encode_rs384);
-	tcase_add_test(tc_core, test_jwt_verify_rs384);
-	tcase_add_test(tc_core, test_jwt_encode_rs512);
-	tcase_add_test(tc_core, test_jwt_verify_rs512);
-	tcase_add_test(tc_core, test_jwt_verify_rsa_i37);
-	tcase_add_test(tc_core, test_jwt_encode_rsa_with_ec);
-	tcase_add_test(tc_core, test_jwt_verify_invalid_token);
-	tcase_add_test(tc_core, test_jwt_verify_invalid_alg);
-	tcase_add_test(tc_core, test_jwt_verify_invalid_cert);
-	tcase_add_test(tc_core, test_jwt_verify_invalid_cert_file);
-	tcase_add_test(tc_core, test_jwt_encode_invalid_key);
+	tcase_add_loop_test(tc_core, test_jwt_encode_rs256, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_rs256, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_validate_rs256, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_encode_rs384, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_rs384, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_encode_rs512, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_rs512, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_rsa_i37, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_encode_rsa_with_ec, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_invalid_token, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_invalid_alg, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_invalid_cert, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_verify_invalid_cert_file, 0, i);
+	tcase_add_loop_test(tc_core, test_jwt_encode_invalid_key, 0, i);
 
 	tcase_set_timeout(tc_core, 120);
 
@@ -400,16 +316,5 @@ static Suite *libjwt_suite(void)
 
 int main(int argc, char *argv[])
 {
-	int number_failed;
-	Suite *s;
-	SRunner *sr;
-
-	s = libjwt_suite();
-	sr = srunner_create(s);
-
-	srunner_run_all(sr, CK_VERBOSE);
-	number_failed = srunner_ntests_failed(sr);
-	srunner_free(sr);
-
-	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+	JWT_TEST_MAIN("LibJWT RSA Sign/Verify");
 }
